
function [Xopt,Popt,Goptimal]=CE_OPT(low_X,up_X,low_P,up_P,Lalpha,Ualpha,Lbeta,Ubeta,alphap,alphaq,betap,betaq,ini_X,ini_P,objectives,lambda,scale,tau,c_opt)
% source code that implement the optimal design algorithms, which include
% both a global and a local search phases

K=size(low_X,1);
N=2000; %400*K;
Nelite=20;

mu0_X=rand(K,1);
sigma0_X=1000*ones(K,1);

tmp_X=up_X-low_X;
index_X=find(tmp_X<=100); 
mu0_X(index_X)=low_X(index_X)+tmp_X(index_X)./2;
sigma0_X(index_X)=10*(tmp_X(index_X).^2);
clear tmp_X; clear index_X;

if ~isempty(ini_X)
    mu0_X=ini_X';
end

mu_X = mu0_X; sigma_X =sigma0_X;

mu0_P=(up_P-low_P)./2;
sigma0_P= 20*ones(K,1);
mu_P=mu0_P; sigma_P=sigma0_P;

k=0;
counter=0;
Gopt=inf;

disp(['Global search ...'])

while (max(sigma_X)>=1e-5 | max(sigma_P)>=1e-10) & k<=250 & counter<=50
    k=k+1;
    %scoef=3/((k+50).^0.51); 
    scoef=0.2;
    % generate random samples from the current distribution
    X=normt_rnd(repmat(mu_X,N,1),repmat(sigma_X,N,1),repmat(low_X,N,1),repmat(up_X,N,1));
    X=reshape(X,K,N)';
    P=normt_rnd(repmat(mu_P,N,1),repmat(sigma_P,N,1),repmat(low_P,N,1),repmat(up_P,N,1));
    P=reshape(P,K,N)';  
    
    % evaluate randomly generated solutions
    G=obj_fun_pm(X,P,lambda,Lalpha,Ualpha,Lbeta,Ubeta,alphap,alphaq,betap,betaq,objectives, scale,tau,c_opt);
    
    [G_sort, I_sort]=sort(G);
    
    if G_sort(1)<=Gopt
        counter=0;
        Gopt=G_sort(1); Xopt=X(I_sort(1),:); Popt=P(I_sort(1),:);
    else
        counter=counter+1;
    end
    % select elite samples
    X_elite=X(I_sort(1:Nelite),:); % N x K matrix
    P_elite=P(I_sort(1:Nelite),:);
    
    % parameter update
    mu_X_old=mu_X; 
    mu_P_old=mu_P;
    mu_X=scoef*mean(X_elite,1)'+(1-scoef)*mu_X;
    mu_P=scoef*mean(P_elite)'+(1-scoef)*mu_P;
    
      
    sigma_X=scoef*var(X_elite)'+(1-scoef)*(sigma_X+(mu_X-mu_X_old).^2);
    sigma_P=scoef*var(P_elite)'+(1-scoef)*(sigma_P+(mu_P-mu_P_old).^2);
    clear X_elite; clear P_elite; 
    clear G; clear G_sort; clear I_sort;
    
    
    if mod(k,10)==0 % print each 10 iterations
        fprintf('%d  %8.5e  ', k, Gopt);
        fprintf('\n');
    end
end

clear X; clear P;

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
disp(['Local search ...'])
% neighborhood search
counter=0; stepsize_X=1e-1; stepsize_P=1e-2;
I_p_equal=find(up_P==low_P); %num_equal=size(I_p_equal,1);
j=0;
while counter<=200 & j<=250*K
    j=j+1;
    delta_X=sign(rand(K,1)-0.5).*stepsize_X;
    X=Xopt'+delta_X; %column vector
    I_x=find(X>=up_X); X(I_x)=up_X(I_x); clear I_x;
    I_x=find(X<=low_X); X(I_x)=low_X(I_x); clear I_x;
    
    I_p_greater=find(Popt'==low_P & up_P~=low_P); %num_greater=size(I_p_greater,1);
    I_p_smaller=find(Popt'==up_P & up_P~=low_P); %num_smaller=size(I_p_smaller,1);
    
    %N_tmp=floor((K-num_equal-num_greater-num_smaller)/2);
    delta_P=zeros(K,1); v_tmp=[1:K]'; v_tmp([I_p_equal; I_p_greater; I_p_smaller])=[];
    
    if ~isempty(v_tmp)
        index_ones=randperm(size(v_tmp,1));
        N_tmp=floor(size(index_ones,2)/2);
        delta_P(v_tmp(index_ones(1:N_tmp)))=1; 
        delta_P(v_tmp(index_ones(N_tmp+1:2*N_tmp)))=-1;
        stepsize_P=min(min(stepsize_P, min(Popt(v_tmp)'-low_P(v_tmp))),min(up_P(v_tmp)-Popt(v_tmp)'));
        P=Popt'+delta_P*stepsize_P;
    else
        P=Popt';
    end
  
    G=obj_fun_pm(X',P',lambda,Lalpha,Ualpha,Lbeta,Ubeta,alphap,alphaq,betap,betaq,objectives, scale,tau,c_opt);
    if G < Gopt
        Gopt=G; Xopt=X'; Popt=P'; counter=0;
    else
        counter=counter+1;
    end
    if mod(j,50*K)==0 % print each 50K iterations
        fprintf('%d  %8.5e  ', j, Gopt);
        fprintf('\n');
    end
end
clear stepsize_X; clear stepsize_P; clear v_tmp; clear delta_P; clear delta_X;
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% a scaled version of SPSA 
ak=1e-3; ck=1e-3; %constant stepsize
stepsize_P=ak; stepsize_Pg=ck; counter=0; 
while counter <=200 & j <=500*K
    j=j+1;
    deltaX=2*round(rand(K,1))-1;
    Xplus=Xopt'+ck*deltaX;
    Xminus=Xopt'-ck*deltaX;
    
    I_p_greater=find(Popt'==low_P & up_P~=low_P); 
    I_p_smaller=find(Popt'==up_P & up_P~=low_P); 
    
    deltaP=zeros(K,1); v_tmp=[1:K]'; v_tmp([I_p_equal; I_p_greater; I_p_smaller])=[];
    
    if ~isempty(v_tmp)
        index_ones=randperm(size(v_tmp,1));
        N_tmp=floor(size(index_ones,2)/2);
        deltaP(v_tmp(index_ones(1:N_tmp)))=1; 
        deltaP(v_tmp(index_ones(N_tmp+1:2*N_tmp)))=-1;
        stepsize_Pg=min(min(stepsize_Pg, min(Popt(v_tmp)'-low_P(v_tmp))),min(up_P(v_tmp)-Popt(v_tmp)'));
        stepsize_P=min(min(stepsize_P, min(Popt(v_tmp)'-low_P(v_tmp))),min(up_P(v_tmp)-Popt(v_tmp)'));
        Pplus=Popt'+deltaP*stepsize_Pg;
        Pminus=Popt'-deltaP*stepsize_Pg;
    else
        Pplus=Popt'; Pminus=Popt';
    end
    I_p=find(deltaP==0); deltaP(I_p)=inf; clear I_p;
    
    Gplus=obj_fun_pm(Xplus',Pplus',lambda,Lalpha,Ualpha,Lbeta,Ubeta,alphap,alphaq,betap,betaq,objectives,scale,tau,c_opt);
    Gminus=obj_fun_pm(Xminus',Pminus',lambda,Lalpha,Ualpha,Lbeta,Ubeta,alphap,alphaq,betap,betaq,objectives,scale,tau,c_opt);
    
    delta_G_X=sign((Gplus-Gminus)./(2*ck*deltaX));
    delta_G_P=sign((Gplus-Gminus)./(2*stepsize_Pg*deltaP));
    X=Xopt'-ak*delta_G_X;
    P=Popt'-stepsize_P*delta_G_P;
    clear delta_G_P; clear delta_G_X;
    
    I_x=find(X>=up_X); X(I_x)=up_X(I_x); clear I_x;
    I_x=find(X<=low_X); X(I_x)=low_X(I_x); clear I_x;
    I_p=find(P>=up_P); P(I_p)=up_P(I_p); clear I_p;
    I_p=find(P<=low_P); P(I_p)=low_P(I_p); clear I_p;
    
    G=obj_fun_pm(X',P',lambda,Lalpha,Ualpha,Lbeta,Ubeta,alphap,alphaq,betap,betaq,objectives,scale,tau,c_opt);
    if G < Gopt 
        Gopt=G; Xopt=X'; Popt=P'; counter=0;
    else
        counter=counter+1;
    end
    if mod(j,50*K)==0
        fprintf('%d  %8.5e  ', j, Gopt);
        fprintf('\n');
    end
end
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
Goptimal=obj_funm(Xopt,Popt,lambda,Lalpha,Ualpha,Lbeta,Ubeta,alphap,alphaq,betap,betaq,objectives,scale,tau,c_opt);