%the main design program that allows a number of options in the choices of
%does ranges, dose levels, dose proportions, design criterion, and potential
%missing trial information. The program calls the optimization algorithm
%CE_OPT.m

function main_design

warning off
flag_K=[];
while isempty(flag_K);
    disp('-----------------------------------------------------');
    disp('Options:');
    disp('(1) specify the number of dose levels');
    disp('(2) calculate the optimal number of dose levels');
    disp('(0) quit the program');
    disp('-----------------------------------------------------');
    flag_K=input('Enter the option: \n'); 
end

if flag_K==1
    K=input('Enter the number of dose levels (default=2): \n');
    if isempty(K)
        K=2;
    end
    doserange=[0 inf]; %default value
    low_P=zeros(K,1); up_P=ones(K,1);
    low_X=ones(K,1)*doserange(1); up_X=ones(K,1)*doserange(2); ini_X=[]; ini_P=[]; alphap=1; alphaq=1; betap=1; betaq=1;
    Ualpha=26; Lalpha=24; Ubeta=8; Lbeta=6; Xopt=[]; Popt=[]; inner_loop=1; weights_obj=[]; num_obj=[]; objectives=[];
    choice=17; tau=1; c_opt=1;
    while choice~=0
        fprintf('\n');
        disp('-----------------------------------------------------');
        disp('Options: ');
        disp('(1) specify the dose range ');
        disp('(2) specify dose levels (e.g., placebo) ');
        disp('(3) specify dose proportions ');
        disp('(4) specify an initial design (optional)');
        disp('(5) specify the initial prior ');
        disp('(6) select the optimality criterion ');
        disp('(7) select the design criterion ');
        disp('(8) specify potential missing trial information ');
        disp('(9) specify the number of iterations ');
        disp('(10) calculate the optimal design');
        disp('(11) evaluate the criterion value for the current design');
        disp('(12) print out the current values');
        disp('(0) quit the program');
        disp('-----------------------------------------------------');
        choice=input('Enter the option: ');
        if choice==1
            disp('------------------------------------------');
            disp('(1) specify the dose range (e.g., [0 60]), default=[0 inf]');
            disp('(2) go back to main menu');
            disp('------------------------------------------');
            rangechoice=input('Enter the choice: \n ');
            if rangechoice==1
                doserange=input('Enter the dose range: \n');
                low_X=ones(K,1)*doserange(1); up_X=ones(K,1)*doserange(2);
            else
                ;
            end
        end
        
        if choice==2
            fprintf('\n');
            disp('-----------------------------------------------------');
            disp('Specify dose levels, examples: ')
            disp('[2 0 30] indicates that dose level 2 is between 0 and 30; ');
            disp('[1 0 0] indicates that dose level 1 is 0 (i.e., placebo); ');
            disp('[3 4 inf] indicates that dose level 3 is above 4;');
            disp('[2 -inf, 50] indicates that dose level 2 is below 50 ');
            disp('-----------------------------------------------------');
            dose_level=input('dose level: \n');
            low_X(dose_level(1))=max(doserange(1),dose_level(2));
            up_X(dose_level(1))=min(doserange(2),dose_level(3));
            
            clear dose_level;
        end
        
        if choice==3
            fprintf('\n');
            disp('-----------------------------------------------------');
            disp('Specify dose proportions, examples: ')
            disp('[2 0 30] indicates that dose level 2 is between 0 and 30%; ');
            disp('[1 10 10] indicates that dose level 1 is exactly 10% ; ');
            disp('[3 40 inf] indicates that dose level 3 is more than 40%;');
            disp('[2 -inf 50] indicates that dose level 2 is below 50% ');
            disp('-----------------------------------------------------');
            dose_pro=input('Dose proportions: \n');
            low_P(dose_pro(1))=max(0,dose_pro(2)/100); 
            up_P(dose_pro(1))=min(1,dose_pro(3)/100);
            clear dose_pro;
        end
        
        if choice==4
            fprintf('\n');
            disp('Available options:');
            disp('-----------------------------------------------------');
            disp('(1) enter an initial design');
            disp('(2) go back to main menu');
            disp('-----------------------------------------------------');
            initialchoice=input('enter the choice: \n');
            if initialchoice==1
                ini_X=input('Enter the support points (e.g.,[0 3 5 9]):\n');
                ini_P=input('Enter the relative weight for each point (e.g., [0.2 0.3 0.2 0.3]):\n');
                ini_P=ini_P/sum(ini_P);
                while size(ini_X,2)~=K | size(ini_P)~=K
                    fprintf('\n');
                    disp('Warning: the number of dose levels does not match!!');
                    disp('Reenter the initial design');
                    ini_X=input('Enter the support points (e.g.,[0 3 5 9]):\n');
                    ini_P=input('Enter the relative weight for each point (e.g., [0.2 0.3 0.2 0.3]):\n');
                    ini_P=ini_P./sum(ini_P);
                end
                I_violatexl=find(ini_X' < low_X); I_violatexu=find(ini_X' > up_X);
                I_violatepl=find(ini_P' < low_P); I_violatepu=find(ini_P' > up_P);
                while ~isempty(I_violatexl) | ~isempty(I_violatexu) | ~isempty(I_violatepl) | ~isempty(I_violatepu)
                    disp('Error: initial design violates design constraints !!');
                    disp('Reenter the initial design');
                    ini_X=input('Enter the support points (e.g.,[0 3 5 9]):\n');
                    ini_P=input('Enter the relative weight for each point (e.g., [0.2 0.3 0.2 0.3]):\n');
                    ini_P=ini_P./sum(ini_P);
                    I_violatexl=find(ini_X' < low_X); I_violatexu=find(ini_X' > up_X);
                    I_violatepl=find(ini_P' < low_P); I_violatepu=find(ini_P' > up_P);
                end
                clear I_violatexl; clear I_violatexu; clear I_violatepl; clear I_violatepu;
            else
                ;
            end
        end
        
        if choice==5
            fprintf('\n');
            disp('-----------------------------------------------------');
            disp('Available options:');
            disp('(1) independent beta distributions for ED50 and slope');
            disp('(2) go back to main menu');
            disp('-----------------------------------------------------');
            dist_choice=input('enter the choice: \n');
            if dist_choice==1
                range_alpha=input('Enter lower and upper bounds for ED50 (e.g.,[4,6]; default [24 26]): \n');
                alphapq=input('Enter the two beta distribution parameters for ED50 (default [1,1]: \n');
                range_beta=input('Enter lower and upper bounds for slope (e.g.,[5 8]; default [6 8]): \n');
                betapq=input('Enter the two beta distribution parameters for the slope (default [1,1]: \n');
                if ~isempty(range_alpha)
                    Ualpha=range_alpha(2); Lalpha=range_alpha(1);
                end
                if ~isempty(alphapq)
                    alphap=alphapq(1); alphaq=alphapq(2);
                end
                if ~isempty(range_beta)
                    Ubeta=range_beta(2); Lbeta=range_beta(1); 
                end
                if ~isempty(betapq)
                    betap=betapq(1); betaq=betapq(2);
                end
            elseif dist_choice==2
                ;
            end
        end
        
        if choice==6
            fprintf('\n');
            disp('-----------------------------------------------------');
            disp('Available options:');
            disp('(1) standardized compound C-optimal design');
            disp('(2) standardized compound C- and D-optimal design');
            disp('(3) non-standardized compound C-optimal design');
            disp('(4) non-standardized compound C- and D-optimal design');
            disp('(5) go back to main menu');
            disp('-----------------------------------------------------');
            opt_choice=input('Enter the choice: \n');
            if opt_choice==1
                c_opt=1; standz=1; % c optimal design , standardize=yes
            elseif opt_choice==2
                c_opt=0; standz=1; %
            elseif opt_choice==3
                c_opt=1; standz=0;
            elseif opt_choice==4
                c_opt=0; standz=0;
            else
                ;
            end
        end
        
        if choice==7
            fprintf('\n');
            disp('-----------------------------------------------------');
            disp('Available options:');
            disp('(1) specify a design criteria');
            disp('(2) go back to main menu');
            disp('-----------------------------------------------------');
            criteria_choice=input('Enter the choice: \n');
            if criteria_choice==1
                if c_opt==1
                    disp('-----------------------------------------------------');
                    disp('Enter the additional criteria other than the slope, examples:');
                    disp('[50] indicates the weighted average of ED50 and slope ');
                    disp('[10 25 75] indicates the weighted average of ED10, ED25, ED75, and slope ');
                    disp('-----------------------------------------------------');
                elseif c_opt==0
                    disp('-----------------------------------------------------');
                    disp('Enter the additional criteria other than the determinant, examples:');
                    disp('[50] indicates the weighted average of ED50 and the determinant ');
                    disp('[10 25 75] indicates the weighted average of ED10, ED25, ED75, and the determinant ');
                    disp('-----------------------------------------------------');
                end
                objectives=input('Enter additional criteria: \n');
                objectives=objectives/100;
                if c_opt==1
                    disp('-----------------------------------------------------');
                    disp('Enter the corresponding weights for the specified criteria and the slope, ');
                    disp('[1 1 1 1] indicates equal weights for a four-objective design (including the slope) ');
                    disp('-----------------------------------------------------');
                elseif c_opt==0
                    disp('-----------------------------------------------------');
                    disp('Enter the corresponding weights for the specified criteria and the determinant, ');
                    disp('[1 1 1 1] indicates equal weights for a four-objective design (including the determinant) ');
                    disp('-----------------------------------------------------');
                end
                weights_obj=input('Enter the weights: \n');
                while size(weights_obj,2) ~= (size(objectives,2)+1)
                    disp('Number of weights does not match the number of objectives !!');
                    weights_obj=input('Re-enter the weights: \n');
                end
                weights_obj=weights_obj/sum(weights_obj);
                num_obj=size(weights_obj,2);
            elseif criteria_choice==2
                ;
            end
        end
        
        if choice==8
            fprintf('\n');
            disp('-----------------------------------------------------');
            disp('Available options:');
            disp('(1) no missing observations');
            disp('(2) logistic non-missing probability function');
            disp('(3) exponential non-missing probability function');
            disp('(4) symmetric non-missing probability function');
            disp('(5) go back to main menu');
            disp('-----------------------------------------------------');
            tau_choice=input('Enter the choice: \n');
            if tau_choice==1
                tau=1;
            elseif tau_choice==2
                tau=2;
                tau_tmp=input('Enter the two parameters [a,b] for the logistic model: \n');
                while isempty(tau_tmp)
                    tau_tmp=input('Enter the two parameters [a,b] for the logistic model: \n');
                end
                tau=[tau tau_tmp]; clear tau_tmp;
            elseif tau_choice==3
                tau=3;
                tau_tmp=input('Enter the three parameters [a,b,c] for the exponential model: \n');
                while isempty(tau_tmp)
                    tau_tmp=input('Enter the three parameters [a,b,c] for the exponential model: \n');
                end
                disp('Note that all function values outside of the range [0,1] will be truncated.');
                tau=[tau tau_tmp]; clear tau_tmp;
            elseif tau_choice==4
                tau=4;
                tau_tmp=input('Enter the three parameters [a,b,c] for the symmetric model: \n');
                while isempty(tau_tmp)
                    tau_tmp=input('Enter the three parameters [a,b,c] for the symmetric model: \n');
                end
                disp('Note that all function values outside of the range [0,1] will be truncated.');
                tau=[tau tau_tmp]; clear tau_tmp;
            elseif tau_choice==5
                ;
            end
        end
        
        if choice==9
            fprintf('\n');
            disp('-----------------------------------------------------');
            disp('Specify the number of iterations (default=1) ')
            disp('More iterations increase the quality of the solution ')
            disp('-----------------------------------------------------');
            inner_loop=input('Number of iterations: \n');
            if isempty(inner_loop)
                inner_loop=1;
            end
        end
        
        if choice==10
            if ~isempty(num_obj)
                Icheck1=find(low_X>up_X); Icheck2=find(low_P>up_P);
                if ~isempty(Icheck1) | ~isempty(Icheck2)
                    disp('Invalid design specification, check the constraint setting!');
                    continue;
                end
                Gopt=0;
                while Gopt < 0.999
                    Gobj_best=ones(1,num_obj)*inf; scale=ones(1,num_obj);
                    lambda_m=eye(num_obj);
                    
                    for h=1:num_obj
                        for l=1:inner_loop
                            [Xopt,Popt,Gobj]=CE_OPT(low_X,up_X,low_P,up_P,Lalpha,Ualpha,Lbeta,Ubeta,alphap,alphaq,betap,betaq,ini_X,ini_P,objectives,lambda_m(h,:),scale,tau,c_opt);
                            if Gobj_best(h) > Gobj
                                Gobj_best(h)=Gobj;
                            end
                        end
                        clear Gobj;
                    end
                    
                    clear Xopt; clear Popt;
                    Gopt=inf;
                    for l=1:inner_loop
                        if standz==1
                            [X,P,G]=CE_OPT(low_X,up_X,low_P,up_P,Lalpha,Ualpha,Lbeta,Ubeta,alphap,alphaq,betap,betaq,ini_X,ini_P,objectives,weights_obj,Gobj_best,tau,c_opt);
                        elseif standz==0
                            [X,P,G]=CE_OPT(low_X,up_X,low_P,up_P,Lalpha,Ualpha,Lbeta,Ubeta,alphap,alphaq,betap,betaq,ini_X,ini_P,objectives,weights_obj,scale,tau,c_opt);
                        end
                        if Gopt > G
                            Gopt=G;
                            Xopt=X; Popt=P;
                        end
                    end
                    if standz==0
                        break;
                    end
                    
                end            
                
                disp('***************************** Output **********************************');
                fprintf('Number of design points: %d \n', K);
                fprintf('X-values: \n');
                fprintf('%8.3f  ',Xopt); fprintf('\n');
                fprintf('Weights: \n');
                fprintf('%8.2f   ',Popt);
                fprintf('\n');
                fprintf('The value of the criterion: %10.8e \n', Gopt);
                disp('-----------------------------------------------------');
                for i=1:num_obj
                    T=num2str(i);
                    fprintf('Objective function %s evaluated at the current design: ', T);
                    fprintf('%10.8e \n',obj_funm(Xopt,Popt,lambda_m(i,:),Lalpha,Ualpha,Lbeta,Ubeta,alphap,alphaq,betap,betaq,objectives,scale,tau,c_opt));
                    fprintf('The estimated optimal value of objective function %s: ', T);
                    fprintf('%10.8e \n',Gobj_best(i));
                end
                disp('-----------------------------------------------------');
                disp('***********************************************************************');
            else
                disp('Specify a design criterion first !!');
            end
        end
        
        if choice==11
            if ~isempty(Xopt)
                if standz==1
                    Gopt=obj_funm(Xopt,Popt,weights_obj,Lalpha,Ualpha,Lbeta,Ubeta,alphap,alphaq,betap,betaq,objectives,Gobj_best,tau,c_opt);
                elseif standz==0
                    Gopt=obj_funm(Xopt,Popt,weights_obj,Lalpha,Ualpha,Lbeta,Ubeta,alphap,alphaq,betap,betaq,objectives,scale,tau,c_opt);
                end
                disp('***************************** Calculated Design **********************************');
                fprintf('Number of design points: %d \n', K);
                fprintf('X-values: \n');
                fprintf('%8.3f  ',Xopt); fprintf('\n');
                fprintf('Weights: \n');
                fprintf('%8.2f   ',Popt);
                fprintf('\n');
                fprintf('The value of the criterion: %10.8e \n', Gopt);
                disp('***********************************************************************');
            end
            if ~isempty(ini_X)
                disp('***************************** Initial Design **********************************');
                fprintf('Number of design points: %d \n', K);
                fprintf('X-values: \n');
                fprintf('%8.3f  ',ini_X); fprintf('\n');
                fprintf('Weights: \n');
                fprintf('%8.2f   ',ini_P);
                fprintf('\n');
                disp('-----------------------------------------------------');
                if ~isempty(objectives)
                    lambda_m=eye(num_obj); scale=ones(1,num_obj);
                    for i=1:num_obj
                        T=num2str(i); 
                        fprintf('Objective function %s evaluated at the current design: ', T);
                        fprintf('%10.8e \n',obj_funm(ini_X,ini_P,lambda_m(i,:),Lalpha,Ualpha,Lbeta,Ubeta,alphap,alphaq,betap,betaq,objectives,scale,tau,c_opt));
                        fprintf('The estimated optimal value of objective function %s: ', T);
                        fprintf('N/A \n');
                    end
                    disp('-----------------------------------------------------');
                else 
                    fprintf('Design criterion not specified !! \n');
                end
                disp('***********************************************************************');
            end
            if isempty(ini_X) & isempty(Xopt)
                disp('No design is available, enter an initial design or calculate a design !!');
            end
            
        end
        
        if choice==12
            disp('**********************************************************************');
            disp('------------------ parameters -----------------------');
            fprintf('Dose range: \n');
            fprintf('%5.4f  %5.4f \n', doserange); fprintf('\n');
            fprintf('Dose level constraints [lower_bound  upper_bound]: \n');
            fprintf('%d : %5.4f  %5.4f \n', [[1:size(low_X,1)]; low_X'; up_X']); fprintf('\n');
            fprintf('Dose proportion constraints [lower_bound  upper_bound]: \n');
            fprintf('%d : %5.4f  %5.4f \n', [[1:size(low_X,1)]; low_P'; up_P']); fprintf('\n');
            fprintf('\n');
            fprintf('Lower and upper bounds for ED50: \n');
            fprintf('%5.4f  %5.4f \n', [Lalpha, Ualpha]);
            fprintf('Two beta distribution parameters for ED50: \n');
            fprintf('%5.4f  %5.4f \n', [alphap, alphaq]);
            fprintf('\n');
            fprintf('Lower and upper bounds for the slope: \n');
            fprintf('%5.4f  %5.4f \n', [Lbeta, Ubeta]);   
            fprintf('Two beta distribution parameters for the slope: \n');
            fprintf('%5.4f  %5.4f \n', [betap, betaq]);  
            fprintf('\n');
            disp('Design criterion: ');
            if ~isempty(objectives)
                if c_opt==1
                    fprintf('weighted variance of '); fprintf(' ED%d,', objectives*100); fprintf(' and the slope. \n');
                elseif c_opt==0
                    fprintf('weighted variance of '); fprintf(' ED%d,', objectives*100); fprintf(' and the determinant. \n');
                end
            else
                fprintf('N/A');
            end
            fprintf('\n');
            fprintf('The weights for different objectives: \n');
            if ~isempty(weights_obj)
                fprintf('%5.4f ', weights_obj);
            else
                fprintf('N/A');
            end
            fprintf('\n');
            disp('Optimality criterion: ');
            if c_opt==1 & standz==1
                fprintf('standardized compound C-optimal design ');
            elseif c_opt==0 & standz==1
                fprintf('standardized compound C- and D-optimal design ');
            elseif c_opt==1 & standz==0
                fprintf('non-standardized compound C-optimal design ');
            elseif c_opt==0 & standz==0
                fprintf('non-standardized compound C- and D-optimal design ');
            end
            fprintf('\n');
            fprintf('Non-missing probability function: \n');
            if tau(1)==1
                fprintf('no missing observations. \n '); 
            elseif tau(1)==2
                fprintf('logistic non-missing probability function with a=%3.2f, b=%3.2f .',tau(2),tau(3));
                fprintf('\n');
            elseif tau(1)==3
                fprintf('exponential non-missing probability function with a=%3.2f, b=%3.2f c=%3.2f .',tau(2),tau(3),tau(4));
                fprintf('\n');
            elseif tau(1)==4
                fprintf('symmetric non-missing probability function with a=%3.2f, b=%3.2f c=%3.2f .',tau(2),tau(3),tau(4));
                fprintf('\n');
            end
            disp('-----------------------------------------------------');
            if ~isempty(Xopt)
                disp('***************************** Calculated Design **********************************');
                if standz==1
                    Gopt=obj_funm(Xopt,Popt,weights_obj,Lalpha,Ualpha,Lbeta,Ubeta,alphap,alphaq,betap,betaq,objectives,Gobj_best,tau,c_opt);
                elseif standz==0
                    Gopt=obj_funm(Xopt,Popt,weights_obj,Lalpha,Ualpha,Lbeta,Ubeta,alphap,alphaq,betap,betaq,objectives,scale,tau,c_opt);
                end
                fprintf('Number of design points: %d \n', K);
                fprintf('X-values: \n');
                fprintf('%8.3f  ',Xopt); fprintf('\n');
                fprintf('Weights: \n');
                fprintf('%8.2f   ',Popt);
                fprintf('\n');
                fprintf('The value of the criterion: %10.8e \n', Gopt);
                disp('-----------------------------------------------------');
                for i=1:num_obj
                    T=num2str(i);
                    fprintf('Objective function %s evaluated at the current design: ', T);
                    fprintf('%10.8e \n',obj_funm(Xopt,Popt,lambda_m(i,:),Lalpha,Ualpha,Lbeta,Ubeta,alphap,alphaq,betap,betaq,objectives,scale,tau,c_opt));
                    fprintf('The estimated optimal value of objective function %s: ', T);
                    fprintf('%10.8e \n',Gobj_best(i));
                end
                disp('-----------------------------------------------------');
            end
            if ~isempty(ini_X)
                disp('***************************** Initial Design **********************************');
                fprintf('Number of design points: %d \n', K);
                fprintf('X-values: \n');
                fprintf('%8.3f  ',ini_X); fprintf('\n');
                fprintf('Weights: \n');
                fprintf('%8.2f   ',ini_P);
                fprintf('\n');
                disp('-----------------------------------------------------');
                if ~isempty(objectives)
                    lambda_m=eye(num_obj); scale=ones(1,num_obj);
                    for i=1:num_obj
                        T=num2str(i);
                        fprintf('Objective function %s evaluated at the current design: ', T);
                        fprintf('%10.8e \n',obj_funm(ini_X,ini_P,lambda_m(i,:),Lalpha,Ualpha,Lbeta,Ubeta,alphap,alphaq,betap,betaq,objectives,scale,tau,c_opt));
                        fprintf('The estimated optimal value of objective function %s: ', T);
                        fprintf('N/A \n');
                    end
                    
                    disp('-----------------------------------------------------');
                else 
                    fprintf('Design criterion not specified !! \n');
                end
            end
            if isempty(ini_X) & isempty(Xopt)
                disp('No design is currently available.');
            end
            disp('**********************************************************************');
            input('Press enter to continue ...');
        end
        
    end
    
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%    
elseif flag_K==2
    doserange=[0 inf]; %default value
    low_P=zeros(2,1); up_P=ones(2,1);
    low_X=ones(2,1)*doserange(1); up_X=ones(2,1)*doserange(2); alphap=1; alphaq=1; betap=1; betaq=1; objectives=[];
    Ualpha=26; Lalpha=24; Ubeta=8; Lbeta=6; dose_pro=[]; dose_level=[]; dose_leveltmp=[]; dose_protmp=[]; num_obj=[];
    ini_X=[]; ini_P=[]; Xopt_current=[]; Popt_current=[]; num_dose_level=2; inner_loop=1; flag_dosenum=0; weights_obj=[];
    choice=17; tau=1; c_opt=1;
    while choice~=0
        fprintf('\n');
        disp('------------------------------------------');
        disp('Options: ');
        disp('(1) specify the dose range ');
        disp('(2) specify the starting number of dose levels ');
        disp('(3) specify dose levels (e.g., placebo) ');
        disp('(4) specify dose proportions ');
        disp('(5) specify the initial prior ');
        disp('(6) select the optimality criterion ');
        disp('(7) select design criteria ');
        disp('(8) specify potential missing trial information ');
        disp('(9) specify the number of iterations for each dose number ');
        disp('(10) calculate the optimal design');
        disp('(11) evaluate the criterion value for the current design');
        disp('(12) print out the current values');
        disp('(0) quit the program');
        disp('------------------------------------------');
        choice=input('Enter the option: ');
        
        if choice==1
            disp('------------------------------------------');
            disp('(1) specify the dose range (e.g., [0 60]), default=[0 inf]');
            disp('(2) go back to main menu');
            disp('------------------------------------------');
            rangechoice=input('Enter the choice: \n ');
            if rangechoice==1
                doserange=input('Enter the dose range: \n');
            else
                ;
            end
        end
        
        if choice==2
            if flag_dosenum==0
                fprintf('\n');
                disp('-----------------------------------------------------');
                disp('(1) specify the starting number of dose levels, default=2 (maximum=6)');
                disp('(2) go back to main menu');
                disp('-----------------------------------------------------');
                numchoice=input('enter the choice: \n ');
                if numchoice==1
                    num_dose_level=input('Starting number of dose levels: \n');
                    if isempty(num_dose_level)
                        num_dose_level=2;
                    end
                else
                    ;
                end
                flag_dosenum=1;
            else
                disp('Cannot change the number of dose levels!');
            end
        end
        
        if choice==3
            fprintf('\n');
            disp('-----------------------------------------------------');
            disp('Specify dose levels, examples: ');
            disp('[2 0 30] indicates that dose level 2 is between 0 and 30; ');
            disp('[1 0 0] indicates that dose level 1 is 0 (i.e., placebo); ');
            disp('[3 4 inf] indicates that dose level 3 is above 4;');
            disp('[2 -inf 50] indicates that dose level 2 is below 50 ');
            disp('-----------------------------------------------------');
            while isempty(dose_leveltmp)
                dose_leveltmp=input('dose level: \n');
            end
            if dose_leveltmp(1) > num_dose_level | dose_leveltmp(1)==0
                disp('Specified dose level dose not exist; ');
                disp('Input ignored!');
                dose_leveltmp=[];
            else
                dose_level(dose_leveltmp(1),:)=dose_leveltmp; dose_leveltmp=[];
            end
        end
        
        if choice==4
            fprintf('\n');
            disp('-----------------------------------------------------');
            disp('Specify dose proportions, examples: ')
            disp('[2 0 30] indicates that dose level 2 is between 0 and 30%; ');
            disp('[1 10 10] indicates that dose level 1 is exactly 10% ; ');
            disp('[2 40 inf] indicates that dose level 2 is more than 40%;');
            disp('-----------------------------------------------------');
            while isempty(dose_protmp)
                dose_protmp=input('Dose proportion: \n');
            end
            if dose_protmp(1) > num_dose_level | dose_protmp(1)==0
                disp('Specified dose level does not exist; ');
                disp('Input ignored!');
                dose_protmp=[];
            else
                dose_pro(dose_protmp(1),:)=dose_protmp; dose_protmp=[];
            end
        end
        
        if choice==5
            fprintf('\n');
            disp('-----------------------------------------------------');
            disp('Available options:');
            disp('(1) independent beta distributions for ED50 and slope');
            disp('(2) go back to main menu');
            disp('-----------------------------------------------------');
            dist_choice=input('Enter the choice: \n');
            if dist_choice==1
                range_alpha=input('Enter lower and upper bounds for ED50 (e.g.,[4,6]; default [24 26]): \n');
                alphapq=input('Enter the two beta distribution parameters for ED50 (default [1,1]: \n');
                range_beta=input('Enter lower and upper bounds for slope (e.g.,[5 8]; default [6 8]): \n');
                betapq=input('Enter the two beta distribution parameters for the slope (default [1,1]: \n');
                if ~isempty(range_alpha)
                    Ualpha=range_alpha(2); Lalpha=range_alpha(1);
                end
                if ~isempty(alphapq)
                    alphap=alphapq(1); alphaq=alphapq(2);
                end
                if ~isempty(range_beta)
                    Ubeta=range_beta(2); Lbeta=range_beta(1); 
                end
                if ~isempty(betapq)
                    betap=betapq(1); betaq=betapq(2);
                end
            elseif dist_choice==2
                ;
            end
        end
        
        if choice==6
            fprintf('\n');
            disp('-----------------------------------------------------');
            disp('Available options:');
            disp('(1) standardized compound C-optimal design');
            disp('(2) standardized compound C- and D-optimal design');
            disp('(3) non-standardized compound C-optimal design');
            disp('(4) non-standardized compound C- and D-optimal design');
            disp('(5) go back to main menu');
            disp('-----------------------------------------------------');
            opt_choice=input('Enter the choice: \n');
            if opt_choice==1
                c_opt=1; standz=1; % c optimal design , standardize=yes
            elseif opt_choice==2
                c_opt=0; standz=1; %
            elseif opt_choice==3
                c_opt=1; standz=0;
            elseif opt_choice==4
                c_opt=0; standz=0;
            else
                ;
            end
        end
        
        if choice==7
            fprintf('\n');
            disp('-----------------------------------------------------');
            disp('Available options:');
            disp('(1) specify the design criteria');
            disp('(2) go back to main menu');
            disp('-----------------------------------------------------');
            criteria_choice=input('enter the choice: \n');
            if criteria_choice==1
                if c_opt==1
                    disp('-----------------------------------------------------');
                    disp('Enter the additional criteria other than the slope, examples:');
                    disp('[50] indicates the weighted average of ED50 and slope ');
                    disp('[10 25 75] indicates the weighted average of ED10, ED25, ED75, and slope ');
                    disp('-----------------------------------------------------');
                elseif c_opt==0
                    disp('-----------------------------------------------------');
                    disp('Enter the additional criteria other than the determinant, examples:');
                    disp('[50] indicates the weighted average of ED50 and determinant ');
                    disp('[10 25 75] indicates the weighted average of ED10, ED25, ED75, and the determinant ');
                    disp('-----------------------------------------------------');
                end
                objectives=input('enter additional criteria: \n');
                objectives=objectives/100;
                if c_opt==1
                    disp('-----------------------------------------------------');
                    disp('Enter the corresponding weights for the specified criteria and the slope, ');
                    disp('[1 1 1 1] indicates equal weights for a four-objective design (including the slope) ');
                    disp('-----------------------------------------------------');
                elseif c_opt==0
                    disp('-----------------------------------------------------');
                    disp('Enter the corresponding weights for the specified criteria and the determinant, ');
                    disp('[1 1 1 1] indicates equal weights for a four-objective design (including the determinant) ');
                    disp('-----------------------------------------------------');
                end
                weights_obj=input('enter the weights: \n');
                while size(weights_obj,2) ~= (size(objectives,2)+1)
                    disp('Number of weights does not match the number of objectives !!');
                    weights_obj=input('Re-enter the weights: \n');
                end
                weights_obj=weights_obj./sum(weights_obj);
                num_obj=size(weights_obj,2);
            elseif criteria_choice==2
                ;
            end
        end
        
        if choice==8
            fprintf('\n');
            disp('-----------------------------------------------------');
            disp('Available options:');
            disp('(1) no missing observations');
            disp('(2) logistic non-missing probability function');
            disp('(3) exponential non-missing probability function');
            disp('(4) symmetric non-missing probability function');
            disp('(5) go back to main menu');
            disp('-----------------------------------------------------');
            tau_choice=input('Enter the choice: \n');
            if tau_choice==1
                tau=1;
            elseif tau_choice==2
                tau=2;
                tau_tmp=input('Enter the two parameters [a,b] for the logistic model: \n');
                while isempty(tau_tmp)
                    tau_tmp=input('Enter the two parameters [a,b] for the logistic model: \n');
                end
                tau=[tau tau_tmp]; clear tau_tmp;
            elseif tau_choice==3
                tau=3;
                tau_tmp=input('Enter the three parameters [a,b,c] for the exponential model: \n');
                while isempty(tau_tmp)
                    tau_tmp=input('Enter the three parameters [a,b,c] for the exponential model: \n');
                end
                disp('Note that all function values outside of the range [0,1] will be truncated. ');
                tau=[tau tau_tmp]; clear tau_tmp;
            elseif tau_choice==4
                tau=4;
                tau_tmp=input('Enter the three parameters [a,b,c] for the symmetric model: \n');
                while isempty(tau_tmp)
                    tau_tmp=input('Enter the three parameters [a,b,c] for the symmetric model: \n');
                end
                disp('Note that all function values outside of the range [0,1] will be truncated.');
                tau=[tau tau_tmp]; clear tau_tmp;
            elseif tau_choice==5
                ;
            end
        end
        
        if choice==9
            fprintf('\n');
            disp('-----------------------------------------------------');
            disp('Specify the number of iterations for each dose number (default=1) ')
            disp('More iterations increase the quality of the solution ')
            disp('-----------------------------------------------------');
            inner_loop=input('Number of iterations: \n');
            if isempty(inner_loop)
                inner_loop=1;
            end
        end
        
        if choice==10
            if ~isempty(num_obj)
                Gopt_current=inf; 
                for K=num_dose_level:6
                    low_P=zeros(K,1); up_P=ones(K,1);
                    if ~isempty(dose_pro)
                        for i=1:size(dose_pro,1)
                            if dose_pro(i,1)~=0
                                low_P(dose_pro(1))=max(0,dose_pro(2)/100); 
                                up_P(dose_pro(1))=min(1,dose_pro(3)/100);
                            end
                        end
                    end
                    low_X=ones(K,1)*doserange(1); up_X=ones(K,1)*doserange(2);
                    if ~isempty(dose_level)
                        for i=1:size(dose_level,1)
                            if dose_level(i,1)~=0
                                low_X(dose_level(i,1))=max(doserange(1),dose_level(i,2));
                                up_X(dose_level(i,1))=min(doserange(2),dose_level(i,3));
                            end
                        end
                    end
                    Icheck1=find(up_X<low_X); Icheck2=find(up_P<low_P);
                    if ~isempty(Icheck1) | ~isempty(Icheck2)
                        disp('Invalid design specification, check the constraint setting!');
                        break;
                    end 
                    Gopt=0;
                    while Gopt < 0.999
                        Gobj_best=ones(1,num_obj)*inf; scale=ones(1,num_obj);
                        lambda_m=eye(num_obj);
                        for h=1:num_obj
                            for l=1:inner_loop
                                [Xopt,Popt,Gobj]=CE_OPT(low_X,up_X,low_P,up_P,Lalpha,Ualpha,Lbeta,Ubeta,alphap,alphaq,betap,betaq,ini_X,ini_P,objectives,lambda_m(h,:),scale,tau,c_opt);
                                if Gobj_best(h) > Gobj
                                    Gobj_best(h)=Gobj;
                                end
                                clear Xopt, clear Popt; clear Gobj;
                            end
                        end
                        %clear Xopt; clear Popt;
                        Gopt=inf;
                        for l=1:inner_loop
                            if standz==1
                                [X,P,G]=CE_OPT(low_X,up_X,low_P,up_P,Lalpha,Ualpha,Lbeta,Ubeta,alphap,alphaq,betap,betaq,ini_X,ini_P,objectives,weights_obj,Gobj_best,tau,c_opt);
                            elseif standz==0
                                [X,P,G]=CE_OPT(low_X,up_X,low_P,up_P,Lalpha,Ualpha,Lbeta,Ubeta,alphap,alphaq,betap,betaq,ini_X,ini_P,objectives,weights_obj,scale,tau,c_opt);
                            end
                            if Gopt > G
                                Gopt=G;
                                Xopt=X; Popt=P;
                            end
                            clear G; clear X; clear P;
                        end
                        if standz==0
                            break;
                        end
                    end            
                    if Gopt_current > Gopt
                        Gopt_current=Gopt;
                        Xopt_current=Xopt;
                        Popt_current=Popt;
                        Kopt_current=K;
                        Gobj_current=Gobj_best;
                    else
                        break;
                    end
                end
                if isempty(Icheck1) & isempty(Icheck2)
                    disp('************************* Output ********************************');
                    fprintf('Number of design points: %d \n', Kopt_current);
                    fprintf('X-values: \n');
                    fprintf('%8.3f  ',Xopt_current); fprintf('\n');
                    fprintf('Weights: \n');
                    fprintf('%8.2f   ',Popt_current);
                    fprintf('\n');
                    fprintf('The value of the criterion: %10.8e \n', Gopt_current);
                    disp('-----------------------------------------------------');
                    for i=1:num_obj
                        T=num2str(i);
                        fprintf('Objective function %s evaluated at the current design: ', T);
                        fprintf('%10.8e \n',obj_funm(Xopt_current,Popt_current,lambda_m(i,:),Lalpha,Ualpha,Lbeta,Ubeta,alphap,alphaq,betap,betaq,objectives,scale,tau,c_opt));
                        fprintf('The estimated optimal value of objective function %s: ', T);
                        fprintf('%10.8e \n',Gobj_current(i));
                    end
                    disp('-----------------------------------------------------');
                    disp('*****************************************************************');
                end
            else
                disp('Specify a design criterion first !!' );
            end
        end
        
        if choice==11
            if ~isempty(Xopt_current)
                if standz==1
                    Gopt_current=obj_funm(Xopt_current,Popt_current,weights_obj,Lalpha,Ualpha,Lbeta,Ubeta,alphap,alphaq,betap,betaq,objectives,Gobj_current,tau,c_opt);
                elseif standz==0
                    Gopt_current=obj_funm(Xopt_current,Popt_current,weights_obj,Lalpha,Ualpha,Lbeta,Ubeta,alphap,alphaq,betap,betaq,objectives,scale,tau,c_opt);
                end
                disp('***************************** Output **********************************');
                fprintf('Number of design points: %d \n', size(Xopt_current,2));
                fprintf('X-values: \n');
                fprintf('%8.3f  ',Xopt_current); fprintf('\n');
                fprintf('Weights: \n');
                fprintf('%8.2f   ',Popt_current);
                fprintf('\n');
                fprintf('The value of the criterion: %10.8e \n', Gopt_current);
                disp('-----------------------------------------------------');
                for i=1:num_obj
                    T=num2str(i);
                    fprintf('Objective function %s evaluated at the current design: ', T);
                    fprintf('%10.8e \n',obj_funm(Xopt_current,Popt_current,lambda_m(i,:),Lalpha,Ualpha,Lbeta,Ubeta,alphap,alphaq,betap,betaq,objectives,scale,tau,c_opt));
                    fprintf('The estimated optimal value of objective function %s: ', T);
                    fprintf('%10.8e \n',Gobj_current(i));
                end
                disp('-----------------------------------------------------');
                disp('***********************************************************************');
            else
                disp('No design is available, calculate a design !');
            end
            input('press enter to continue ...');
        end
        
        if choice==12
            disp('**********************************************************************');
            disp('------------------ parameters -----------------------');
            fprintf('Dose range: \n');
            fprintf('%5.4f  %5.4f \n', doserange); fprintf('\n');
            fprintf('Dose level constraints [lower_bound  upper_bound]: \n');
            if ~isempty(Xopt_current)
                fprintf('%d : %5.4f  %5.4f \n', [[1:Kopt_current]; low_X(1:Kopt_current)'; up_X(1:Kopt_current)']); fprintf('\n');
                fprintf('Dose proportion constraints [lower_bound  upper_bound]: \n');
                fprintf('%d : %5.4f  %5.4f \n', [[1:Kopt_current];low_P(1:Kopt_current)'; up_P(1:Kopt_current)']); fprintf('\n');
            else
                low_P=zeros(num_dose_level,1); up_P=ones(num_dose_level,1);
                if ~isempty(dose_pro)
                    for i=1:size(dose_pro,1)
                        if dose_pro(i,1)~=0
                            low_P(dose_pro(i,1))=max(0,dose_pro(i,2)/100); 
                            up_P(dose_pro(i,1))=min(1,dose_pro(i,3)/100);
                        end
                    end
                end
                low_X=ones(num_dose_level,1)*doserange(1); up_X=ones(num_dose_level,1)*doserange(2);
                if ~isempty(dose_level)
                    for i=1:size(dose_level,1)
                        if dose_level(i,1)~=0
                            low_X(dose_level(i,1))=max(doserange(1),dose_level(i,2));
                            up_X(dose_level(i,1))=min(doserange(2),dose_level(i,3));
                        end
                    end
                end
                fprintf('%d : %5.4f  %5.4f \n', [[1:size(low_X,1)]; low_X'; up_X']); fprintf('\n');
                fprintf('Dose proportion constraints [lower_bound  upper_bound]: \n');
                fprintf('%d : %5.4f  %5.4f \n', [[1:size(low_X,1)];low_P'; up_P']); fprintf('\n');
            end
            fprintf('Lower and upper bounds for ED50: \n');
            fprintf('%5.4f  %5.4f \n', [Lalpha, Ualpha]);
            fprintf('Two beta distribution parameters for ED50: \n');
            fprintf('%5.4f  %5.4f \n', [alphap, alphaq]);
            fprintf('\n');
            fprintf('Lower and upper bounds for the slope: \n');
            fprintf('%5.4f  %5.4f \n', [Lbeta, Ubeta]);  
            fprintf('Two beta distribution parameters for the slope: \n');
            fprintf('%5.4f  %5.4f \n', [betap, betaq]);
            fprintf('\n');
            disp('Design criterion: ');
            if ~isempty(objectives)
                if c_opt==1 
                    fprintf('weighted variance of '); fprintf(' ED%d,', objectives*100); fprintf(' and the slope. \n');
                elseif c_opt==0
                    fprintf('weighted variance of '); fprintf(' ED%d,', objectives*100); fprintf(' and the determinant. \n');
                end
            else
                fprintf('N/A');
            end
            fprintf('\n');
            fprintf('The weights for different objectives: \n');
            if ~isempty(weights_obj)
                fprintf('%5.4f ', weights_obj);
            else
                fprintf('N/A ');
            end
            fprintf('\n');
            disp('Optimality criterion: ');
            if c_opt==1 & standz==1
                fprintf('standardized compound C-optimal design');
            elseif c_opt==0 & standz==1
                fprintf('standardized compound C- and D-optimal design ');
            elseif c_opt==1 & standz==0
                fprintf('non-standardized compound C-optimal design');
            elseif c_opt==0 & standz==0
                fprintf('non-standardized compound C- and D-optimal design');
            end
            fprintf('\n');
            fprintf('Non-missing probability function: \n');
            if tau(1)==1
                fprintf('no missing observations. \n '); 
            elseif tau(1)==2
                fprintf('logistic non-missing probability function with a=%3.2f, b=%3.2f .',tau(2),tau(3));
                fprintf('\n');
            elseif tau(1)==3
                fprintf('exponential non-missing probability function with a=%3.2f, b=%3.2f c=%3.2f .',tau(2),tau(3),tau(4));
                fprintf('\n');
            elseif tau(1)==4
                fprintf('symmetric non-missing probability function with a=%3.2f, b=%3.2f c=%3.2f .',tau(2),tau(3),tau(4));
                fprintf('\n');
            end
            disp('-----------------------------------------------------');
            if ~isempty(Xopt_current)
                %Gopt=obj_fun(Xopt,Popt,lambda,Lalpha,Ualpha,Lbeta,Ubeta,Galpha_current,Gbeta_current);
                fprintf('Number of design points: %d \n', Kopt_current);
                fprintf('X-values: \n');
                fprintf('%8.3f  ',Xopt_current); fprintf('\n');
                fprintf('Weights: \n');
                fprintf('%8.2f   ',Popt_current);
                fprintf('\n');
                fprintf('The value of the criterion: %10.8e \n', Gopt_current);
                disp('-----------------------------------------------------');
                for i=1:num_obj
                    T=num2str(i);
                    fprintf('Objective function %s evaluated at the current design: ', T);
                    fprintf('%10.8e \n',obj_funm(Xopt_current,Popt_current,lambda_m(i,:),Lalpha,Ualpha,Lbeta,Ubeta,alphap,alphaq,betap,betaq,objectives,scale,tau,c_opt));
                    fprintf('The estimated optimal value of objective function %s: ', T);
                    fprintf('%10.8e \n',Gobj_current(i));
                end
                disp('-----------------------------------------------------');
            else
                disp('No design is currently available.');
            end
            disp('**********************************************************************');
            input('Press enter to continue ... ');
        end
    end
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%    
elseif flag_K==0
    exit;
end