% X: N x K matrix, each row represents a set of design points
% P: N x K matrix, each row represents the corresponding response
% probabilities
%Lalpha, Ualpha : the lower and upper bound for alpha
%Lbeta, Ubeta: the lower and upper bound for beta
%objective function for the compound optimal design criterion

function G=obj_funm(X,P,lambda,Lalpha,Ualpha,Lbeta,Ubeta,alphap,alphaq,betap,betaq,objectives,scale,tau,c_opt)
n_obj=size(scale,2);
N=size(X,1); % number of samples
gamma=log(objectives./(1-objectives));  %row vector;
if tau(1)==1
    tau_V=ones(N,size(X,2));
elseif tau(1)==2
    tau_V=1./(1+exp(tau(2)*X-tau(3)));
elseif tau(1)==3
    tau_V=1-tau(2)*exp(tau(3)*X-tau(4));
    tau_V=tau_V.*(tau_V>=0); tau_V=tau_V.*(tau_V<=1)+(tau_V>1);
elseif tau(1)==4
    tau_V=tau(2)*(tau(3)-X).^2+tau(4);
    tau_V=tau_V.*(tau_V>=0); tau_V=tau_V.*(tau_V<=1)+(tau_V>1);
end

Const=Beta(alphap,alphaq)*Beta(betap,betaq);
% gaussian quadrature
% 5 points
z=[-0.906179846 -0.538469310 0 0.538469310 0.906179846];
weight=[0.236926885 0.478628670 0.568888889 0.478628670 0.236926885];

a_alpha=(Ualpha-Lalpha)/2; b_alpha=(Ualpha+Lalpha)/2; 
a_beta=(Ubeta-Lbeta)/2; b_beta=(Ubeta+Lbeta)/2;
alphal=a_alpha*z+b_alpha; betal=a_beta*z+b_beta;
Const=a_alpha*a_beta/Const;
G=0;
for i=1:size(z,2)
    for j=1:size(z,2)
        W=betal(j)*(X-alphal(i));
        denominator=(1+exp(W)).^2;
        a= sum(P.*tau_V.*exp(W)./denominator,2); % N by 1 column vector;
        b=sum(P.*tau_V.*((X-alphal(i)).^2).*exp(W)./denominator,2);
        c=-sum(P.*tau_V.*(X-alphal(i)).*exp(W)./denominator,2);
        
        for k=1:n_obj-1
            if lambda(k)~=0
                G=G+weight(i)*weight(j)*lambda(k)*(b+2*c*gamma(k)/betal(j)+a*(gamma(k)/betal(j))^2)./(betal(j)^2*(a.*b-c.^2))/scale(k)*(alphal(i)^(alphap-1))*((1-alphal(i))^(alphaq-1))*(betal(j)^(betap-1))*((1-betal(j))^(betaq-1));
            end 
        end
        if c_opt==1
            if lambda(end)~=0
                G=G+weight(i)*weight(j)*lambda(end)*a./(a.*b-c.^2)/scale(end)*(alphal(i)^(alphap-1))*((1-alphal(i))^(alphaq-1))*(betal(j)^(betap-1))*((1-betal(j))^(betaq-1));
            end    
        elseif c_opt==0
            if lambda(end)~=0
                G=G+weight(i)*weight(j)*lambda(end)*(-1)*log(betal(j)^2*(a.*b-c.^2))/scale(end)*(alphal(i)^(alphap-1))*((1-alphal(i))^(alphaq-1))*(betal(j)^(betap-1))*((1-betal(j))^(betaq-1));
            end
        end
    end
end
G=(G*Const);